/* Partial: _coverage.css.j2 — Coverage icons */

.coverage-cell {
    display: flex;
    align-items: center;
    justify-content: center;
    gap: 4px;
}

.coverage-icon {
    font-size: 1rem;
}

.coverage-icon.none { color: #adb5bd; }
.coverage-icon.partial { color: #6c757d; }
.coverage-icon.full { color: #212529; }

.warning-icon {
    color: #ffc107;
    font-size: 0.875rem;
}
