"""
elspais.commands - CLI command implementations
"""

__all__ = [
    "analyze",
    "associate_cmd",
    "changed",
    "completion",
    "config_cmd",
    "edit",
    "example_cmd",
    "fix_cmd",
    "index",
    "init",
    "install_cmd",
    "link_suggest",
    "pdf_cmd",
    "reformat_cmd",
    "rules_cmd",
    "trace",
    "validate",
]
