"""HTML Generation module for traceability reports.

This module provides HTML generation for interactive traceability matrices.
"""

from elspais.html.generator import HTMLGenerator

__all__ = ["HTMLGenerator"]
